package cz.mendelu.adaptive;


import java.util.ArrayList;
import java.util.List;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import cz.mendelu.adaptive.domain.Answer;
import cz.mendelu.adaptive.domain.Exam;
import cz.mendelu.adaptive.domain.ExamResult;
import cz.mendelu.adaptive.domain.GraphChapterNode;
import cz.mendelu.adaptive.domain.Question;
import cz.mendelu.adaptive.services.CustomSecurityUser;
import cz.mendelu.adaptive.services.ExamDaoService;
import cz.mendelu.adaptive.services.UserDaoService;
import cz.mendelu.adaptive.tools.ActiveExamsRepository;

/**
 * Handles requests for the application home page.
 */
@Controller
public class ResultController {
	
	@Autowired
	public ExamDaoService examDaoService;
	
	@Autowired
	public UserDaoService userDaoService;
	
	private static final Logger logger = LoggerFactory.getLogger(ResultController.class);
	
	@RequestMapping(value = "/examDetail/{examId}", method = RequestMethod.GET)
	public ModelAndView examDetailView(@PathVariable("examId") Long examId, Authentication auth) {
		logger.info("Get request on examDetail page. ");
		ModelAndView mv = new ModelAndView("examDetail");
		mv.addObject("userId",((CustomSecurityUser) auth.getPrincipal()).getId());
		mv.addObject("exam", examDaoService.findExamById(examId));
		mv.addObject("chapters", examDaoService.getExamChapterCount(examId));
		return mv;
	}
	
	@RequestMapping(value = "/examDetail/{examId}", method = RequestMethod.POST)
	public String examDetail(@RequestParam("userId") Long userId, @RequestParam("examId") Long examId) {
		logger.info("POST request on takeExamsView page. ");
		Exam exam = examDaoService.findExamById(examId);
		ActiveExamsRepository.addActiveExam(userDaoService.findUserById(userId), exam, examDaoService.findRandomQuestionsByChapterId(exam.getRootNode().getChapterId(), exam.getGenerateQuestions()), exam.getRootNode());
		return "redirect:/examination";
	}

	@RequestMapping(value = "/takeExam", method = RequestMethod.GET)
	public ModelAndView takeExamView(Authentication auth) {
		logger.info("GET request on takeExamsView page. ");		
		
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();
		 
		ModelAndView mv = new ModelAndView("takeExam");
		mv.addObject("classExams", examDaoService.findAvailablePrivateExamsByUserId(userId));
		mv.addObject("exams", examDaoService.findPublicExams());
		return mv;
	}
	
	@RequestMapping(value = "/examination", method = RequestMethod.GET)
	public ModelAndView examinationView(Authentication auth) {
		logger.info("GET request on examination page. ");

		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();

		if (!ActiveExamsRepository.isActive(userId)) 
			return new ModelAndView("exception");
		if (ActiveExamsRepository.isQuestionQueueEmpty(userId)){
		
			examDaoService.createExamResult(ActiveExamsRepository.getExamResult(userId));
			return new ModelAndView("userResultDetail/"+ ActiveExamsRepository.getExamResult(userId).getResultId());
		}
		
		Question question = ActiveExamsRepository.getNextQuestion(userId);
		ModelAndView mv = new ModelAndView("examination");
		
	//	if (ActiveExamsRepository.isChapterQueueEmpty(userId)) return new ModelAndView("result");
		mv.addObject("chapter", ActiveExamsRepository.getNextChapter(userId));
		mv.addObject("question", question);
		mv.addObject("answer", new Answer());
		mv.addObject("options", examDaoService.findOptionsByQuestionId(question.getQuestionId()));
		return mv; 
	}
	
	
	@RequestMapping(value = "/examination", method = RequestMethod.POST)
	public String examination(Authentication auth, @Valid Answer answer) {
		logger.info("POST request on examination page. ");
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();
		
		answer.setCorrect(examDaoService.isOptionCorrectByOptionId(answer.getSelectedId()));
		ActiveExamsRepository.addChapterAnswer(userId, ActiveExamsRepository.getNextChapter(userId), answer);
		ActiveExamsRepository.removeQuestion(userId);
		
		if (ActiveExamsRepository.isQuestionQueueEmpty(userId)){ 
			GraphChapterNode actualChapter = ActiveExamsRepository.getNextChapter(userId);
			
			if (ActiveExamsRepository.isChapterAnsweredCorrectly(userId, actualChapter)){

				List<GraphChapterNode> childChapters = examDaoService.findFirstChapterSuccessorsByParentId(actualChapter.getChapterId());
				ActiveExamsRepository.removeChapter(userId);
				List<GraphChapterNode> addList = new ArrayList<GraphChapterNode>();
				
				for (GraphChapterNode chapter : childChapters) 
					
					if (chapter.isDependant()){
						if (ActiveExamsRepository.hasDoneDependencies(userId, examDaoService.findFirstAncestorsById(chapter.getChapterId())))
							addList.add(chapter);
					}else
						addList.add(chapter);
				ActiveExamsRepository.addChaptersToQueue(userId, addList);
			}else
				ActiveExamsRepository.removeChapter(userId);
		
			if (ActiveExamsRepository.isChapterQueueEmpty(userId)) {

				examDaoService.createExamResult(ActiveExamsRepository.getExamResult(userId));
				return "redirect:/userResultDetail/"+ ActiveExamsRepository.getExamResult(userId).getResultId();
			}
			
			ActiveExamsRepository.addQuestionsToQueue(userId, examDaoService.findRandomQuestionsByChapterId(ActiveExamsRepository.getNextChapter(userId).getChapterId(),ActiveExamsRepository.getGenerateCount(userId)));
		}
		return "redirect:/examination";
	}
	
	@RequestMapping(value = "/resultView", method = RequestMethod.GET)
	public ModelAndView myExamResultsView(Authentication auth) {
		logger.info("Get request on resultView page. ");
		ModelAndView mv = new ModelAndView("resultView");
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();
		mv.addObject("results", examDaoService.findExamResultsByUserId(userId));

		return mv;
	}
	
	@RequestMapping(value = "/examResultDetail/{examResultId}", method = RequestMethod.GET)
	public ModelAndView examResultDetail(Authentication auth, @PathVariable("examResultId") Long examResultId) {
		logger.info("Get request on examResultDetail page. ");
		ModelAndView mv = new ModelAndView("examResultDetail");
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();
		ExamResult result =  examDaoService.findExamResultByResultId(examResultId);
		mv.addObject("chapters", examDaoService.findChapterResultByUserId(userId, examResultId));
		mv.addObject("count", examDaoService.getExamChapterCount(result.getExam().getExamId()));
		mv.addObject("result", result);
		return mv;
	}
	
	@RequestMapping(value = "/userResultDetail/{examResultId}", method = RequestMethod.GET)
	public ModelAndView userResultDetail(Authentication auth, @PathVariable("examResultId") Long examResultId) {
		logger.info("Get request on userResultDetail page. ");
		ModelAndView mv = new ModelAndView("userResultDetail");
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();
		ExamResult result =  examDaoService.findExamResultByResultId(examResultId);
		mv.addObject("chapters", examDaoService.findChapterResultByUserId(userId, examResultId));
		mv.addObject("count", examDaoService.getExamChapterCount(result.getExam().getExamId()));
		mv.addObject("result", result);
		return mv;
	}
	
	@RequestMapping(value = "/browseResults", method = RequestMethod.GET)
	public ModelAndView ownExamResultsView(Authentication auth) {
		logger.info("Get request on ownerResults page. ");
		ModelAndView mv = new ModelAndView("browseResults");
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();
		mv.addObject("results", examDaoService.findExamResultsByOwnerId(userId));

		return mv;
	}

}
